@@ -105,7 +105,7 @@ fuzzing(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr,
     caplen = pkthdr->caplen;
     plugin = tcpedit->dlt_ctx->encoder;
     l2len = plugin->plugin_l2len(ctx, packet, caplen);
-    l2proto = ntohs(ctx->proto);
+    l2proto = ntohs(plugin->plugin_proto(ctx, packet, caplen));
     if (l2len == -1 || caplen < l2len)
         goto done;
 
@@ -307,7 +307,11 @@ fuzzing(tcpedit_t *tcpedit, struct pcap_pkthdr *pkthdr,
     }
 
     /* in cases where 'l3data' is a working buffer, copy it back to '*pkthdr' */
-    plugin->plugin_merge_layer3(ctx, packet, caplen, l3data);
+    plugin->plugin_merge_layer3(ctx,
+                                packet,
+                                caplen,
+                                (l2proto == ETHERTYPE_IP) ? l4data : NULL,
+                                (l2proto == ETHERTYPE_IPV6) ? l4data : NULL);
 
 done:
     return packet_changed;
