@@ -2,7 +2,7 @@
 
 /*
  *   Copyright (c) 2001-2010 Aaron Turner <aturner at synfin dot net>
- *   Copyright (c) 2013-2018 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
+ *   Copyright (c) 2013-2022 Fred Klassen <tcpreplay at appneta dot com> - AppNeta
  *
  *   The Tcpreplay Suite of tools is free software: you can redistribute it 
  *   and/or modify it under the terms of the GNU General Public License as 
@@ -125,18 +125,16 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     
     /* not everything has a L3 header, so check for errors.  returns proto in network byte order */
     if ((l2proto = tcpedit_dlt_proto(tcpedit->dlt_ctx, src_dlt, packet, (*pkthdr)->caplen)) < 0) {
-        dbg(2, "Packet has no L3+ header");
+        dbgx(2, "Packet has no L3+ header: %s", tcpedit_geterr(tcpedit));
+        return TCPEDIT_SOFT_ERROR;
     } else {
         dbgx(2, "Layer 3 protocol type is: 0x%04x", ntohs(l2proto));
     }
         
     /* rewrite Layer 2 */
-    if ((pktlen = tcpedit_dlt_process(tcpedit->dlt_ctx, pktdata, (*pkthdr)->caplen, direction)) == TCPEDIT_ERROR)
-        errx(-1, "%s", tcpedit_geterr(tcpedit));
-
-    /* unable to edit packet, most likely 802.11 management or data QoS frame */
-    if (pktlen == TCPEDIT_SOFT_ERROR) {
-        dbgx(3, "%s", tcpedit_geterr(tcpedit));
+    if ((pktlen = tcpedit_dlt_process(tcpedit->dlt_ctx, pktdata, (*pkthdr)->caplen, direction)) < 0) {
+        /* unable to edit packet, most likely 802.11 management or data QoS frame */
+        dbgx(3, "Failed to edit DLT: %s", tcpedit_geterr(tcpedit));
         return TCPEDIT_SOFT_ERROR;
     }
 
@@ -216,13 +214,14 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         }
 
         /* rewrite the TTL */
-        rewrite_ipv4_ttl(tcpedit, ip_hdr);
+        needtorecalc += rewrite_ipv4_ttl(tcpedit, ip_hdr);
 
         /* rewrite TCP/UDP ports */
         if (tcpedit->portmap != NULL) {
             if ((retval = rewrite_ipv4_ports(tcpedit, &ip_hdr,
                     (*pkthdr)->caplen - l2len)) < 0)
                 return TCPEDIT_ERROR;
+            needtorecalc += retval;
         }
 
         if (tcpedit->tcp_sequence_enable)
@@ -233,7 +232,7 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
     /* IPv6 edits */
     else if (ip6_hdr != NULL) {
         /* rewrite the hop limit */
-        rewrite_ipv6_hlim(tcpedit, ip6_hdr);
+        needtorecalc += rewrite_ipv6_hlim(tcpedit, ip6_hdr);
 
         /* set traffic class? */
         if (tcpedit->tclass > -1) {
@@ -266,6 +265,7 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
             if ((retval = rewrite_ipv6_ports(tcpedit, &ip6_hdr,
                     (*pkthdr)->caplen - l2len)) < 0)
                 return TCPEDIT_ERROR;
+            needtorecalc += retval;
         }
 
         if (tcpedit->tcp_sequence_enable)
@@ -294,10 +294,12 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
             if ((retval = rewrite_ipv4l3(tcpedit, ip_hdr, direction,
                     (*pkthdr)->caplen - l2len)) < 0)
                 return TCPEDIT_ERROR;
+            needtorecalc += retval;
         } else if (ip6_hdr != NULL) {
             if ((retval = rewrite_ipv6l3(tcpedit, ip6_hdr, direction,
                     (*pkthdr)->caplen - l2len)) < 0)
                 return TCPEDIT_ERROR;
+            needtorecalc += retval;
         }
 
         /* ARP packets */
@@ -320,11 +322,13 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
             if ((retval = randomize_ipv4(tcpedit, *pkthdr, packet, 
                     ip_hdr, (*pkthdr)->caplen - l2len)) < 0)
                 return TCPEDIT_ERROR;
+            needtorecalc += retval;
 
         } else if (ip6_hdr != NULL) {
             if ((retval = randomize_ipv6(tcpedit, *pkthdr, packet,
                     ip6_hdr, (*pkthdr)->caplen - l2len)) < 0)
                 return TCPEDIT_ERROR;
+            needtorecalc += retval;
 
         /* ARP packets */
         } else if (l2proto == htons(ETHERTYPE_ARP)) {
@@ -340,16 +344,6 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         }
     }
 
-    /*
-     * fix IP packet lengths in case they are corrupted by TCP segmentation
-     * offload
-     */
-    if (ip_hdr != NULL) {
-        fix_ipv4_length(*pkthdr, ip_hdr);
-    } else if (ip6_hdr != NULL) {
-        fix_ipv6_length(*pkthdr, ip6_hdr);
-    }
-
     /* do we need to fix checksums? -- must always do this last! */
     if ((tcpedit->fixcsum || needtorecalc)) {
         if (ip_hdr != NULL) {
@@ -365,11 +359,15 @@ tcpedit_packet(tcpedit_t *tcpedit, struct pcap_pkthdr **pkthdr,
         if (retval < 0) {
             return TCPEDIT_ERROR;
         } else if (retval == TCPEDIT_WARN) {
-            dbgx(3, "%s", tcpedit_getwarn(tcpedit));
+            warnx("%s", tcpedit_getwarn(tcpedit));
         }
     }
 
-    tcpedit_dlt_merge_l3data(tcpedit->dlt_ctx, dst_dlt, packet, (*pkthdr)->caplen, (u_char *)ip_hdr);
+    tcpedit_dlt_merge_l3data(tcpedit->dlt_ctx,
+                             dst_dlt, packet,
+                             (*pkthdr)->caplen,
+                             (u_char*)ip_hdr,
+                             (u_char*)ip6_hdr);
 
     tcpedit->runtime.total_bytes += (*pkthdr)->caplen;
     tcpedit->runtime.pkts_edited ++;
@@ -407,10 +405,6 @@ tcpedit_init(tcpedit_t **tcpedit_ex, int dlt)
     dbgx(1, "Input file (1) datalink type is %s",
             pcap_datalink_val_to_name(dlt));
 
-#ifdef FORCE_ALIGN
-    tcpedit->runtime.l3buff = (u_char *)safe_malloc(MAXPACKET);
-#endif
-
     return TCPEDIT_OK;
 }
 
@@ -625,11 +619,6 @@ tcpedit_close(tcpedit_t **tcpedit_ex)
         tcpedit->portmap = NULL;
     }
 
-#ifdef FORCE_ALIGN
-    safe_free(tcpedit->runtime.l3buff);
-    tcpedit->runtime.l3buff = NULL;
-#endif
-
     safe_free(*tcpedit_ex);
     *tcpedit_ex = NULL;
 
